import type { Entry } from '@google-cloud/logging';
import { Duration, HttpRequest, Metadata, Resource, ResourceType, ServiceContext } from './types/input';
export declare function createMetadata<T extends Metadata>(metadata: T): T;
export declare function createResource<T extends ResourceType>({ type, labels }: Resource<T>): Resource<T>;
export declare function createServiceContext<T extends ServiceContext>(serviceContext: T): T;
declare type Input = {
    metadata?: Metadata;
    resource?: Resource<any>;
    serviceContext?: ServiceContext;
    message?: string | Error;
    payload?: Record<string, unknown>;
};
export declare function toLoggingClient(projectId: string, input: Input, skipDefaults?: boolean): [metadata: Entry['metadata'], data: Entry['data']];
export declare function toStd(projectId: string, input: Input, skipDefaults?: boolean): {
    timestamp?: string | Date | undefined;
    httpRequest?: (Pick<HttpRequest, "requestMethod" | "requestUrl" | "requestSize" | "status" | "responseSize" | "userAgent" | "remoteIp" | "serverIp" | "referer" | "cacheHit" | "cacheLookup" | "cacheFillBytes" | "protocol" | "cacheValidatedWithOriginServer"> & {
        latency?: Duration | undefined;
    }) | undefined;
    logName?: string | undefined;
    severity?: "DEFAULT" | "DEBUG" | "INFO" | "NOTICE" | "WARNING" | "ERROR" | "CRITICAL" | "ALERT" | "EMERGENCY" | undefined;
    resource: Resource<any> | undefined;
    "logging.googleapis.com/insertId": string | undefined;
    "logging.googleapis.com/labels": Record<string, string> | undefined;
    "logging.googleapis.com/sourceLocation": {
        file: string;
        line: number;
        function?: string | undefined;
    } | undefined;
    "logging.googleapis.com/spanId": string | undefined;
    "logging.googleapis.com/trace": string | undefined;
    "logging.googleapis.com/traceSampled": boolean | undefined;
    "logging.googleapis.com/operation": {
        id?: string | undefined;
        producer?: string | undefined;
        first?: boolean | undefined;
        last?: boolean | undefined;
    } | undefined;
    message: string | undefined;
    serviceContext: ServiceContext | undefined;
};
export declare function toFluentBit130(projectId: string, input: Input, skipDefaults?: boolean): Record<string, any>;
export {};
