"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toFluentBit130 = exports.toStd = exports.toLoggingClient = exports.createServiceContext = exports.createResource = exports.createMetadata = void 0;
const utils_1 = require("./utils");
const eventid_1 = require("eventid");
const constants_1 = require("./constants");
const precise_date_1 = require("@google-cloud/precise-date");
const eventId = new eventid_1.EventId();
function createMetadata(metadata) {
    return metadata;
}
exports.createMetadata = createMetadata;
function createResource({ type, labels }) {
    return {
        type,
        labels,
    };
}
exports.createResource = createResource;
function createServiceContext(serviceContext) {
    return serviceContext;
}
exports.createServiceContext = createServiceContext;
function processMetadata({ projectId, metadata, resource, }, skipDefaults) {
    if (typeof metadata === 'undefined') {
        return {};
    }
    const { httpRequest } = metadata, metadataRest = __rest(metadata, ["httpRequest"]);
    const result = Object.assign(Object.assign({}, metadataRest), { resource });
    if (!skipDefaults) {
        result.insertId || (result.insertId = eventId.new());
        result.severity || (result.severity = constants_1.SEVERITY.DEFAULT);
        result.timestamp || (result.timestamp = new precise_date_1.PreciseDate().toISOString());
    }
    result.logName = utils_1.createFullyQualifiedIdentifier('logs', metadata.logName || 'log', projectId);
    result.trace = utils_1.createFullyQualifiedIdentifier('traces', result.trace, projectId);
    result.httpRequest = utils_1.formatHttpRequest(httpRequest);
    return result;
}
function toLoggingClient(projectId, input, skipDefaults) {
    const { metadata, resource, serviceContext, message, payload } = input;
    return [
        Object.assign({}, processMetadata({ projectId, metadata, resource }, skipDefaults)),
        Object.assign(Object.assign({}, payload), { message: utils_1.formatMessage(message), serviceContext }),
    ];
}
exports.toLoggingClient = toLoggingClient;
function toStd(projectId, input, skipDefaults) {
    const { metadata, resource, serviceContext, message, payload } = input;
    const stdMetadata = utils_1.convertClientMetadataToStdMetadata(processMetadata({ projectId, metadata, resource }, skipDefaults));
    return Object.assign(Object.assign(Object.assign({}, payload), { message: utils_1.formatMessage(message), serviceContext, resource }), stdMetadata);
}
exports.toStd = toStd;
function toFluentBit130(projectId, input, skipDefaults) {
    const output = toStd(projectId, input, skipDefaults);
    const { httpRequest } = output, outputRest = __rest(output, ["httpRequest"]);
    const result = outputRest;
    if (result.httpRequest && typeof (httpRequest === null || httpRequest === void 0 ? void 0 : httpRequest.latency) !== 'undefined') {
        result.httpRequest.latency = utils_1.convertDurationToString(httpRequest.latency);
    }
    return result;
}
exports.toFluentBit130 = toFluentBit130;
//# sourceMappingURL=helpers.js.map