"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeLabels = exports.log = exports.convertDurationToString = exports.formatHttpRequest = exports.createFullyQualifiedIdentifier = exports.validateProjectId = exports.formatMessage = exports.convertErrorToMessage = exports.convertClientMetadataToStdMetadata = void 0;
const constants_1 = require("./constants");
const mathjs_1 = require("mathjs");
/**
 * Converts Stackdriver special fields into their corresponding
 * named keys that can be picked up by fluentd or fluent-bit
 * @param metadata
 */
function convertClientMetadataToStdMetadata(metadata) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const resultingMetadata = Object.assign({}, metadata);
    // set special field keys and remove client keys
    Object.entries(constants_1.CLIENT_KEYS_TO_SPECIAL_STD_KEYS).forEach(([clientKey, stdKey]) => {
        if (clientKey in resultingMetadata) {
            resultingMetadata[stdKey] = resultingMetadata[clientKey];
        }
        delete resultingMetadata[clientKey];
    });
    return resultingMetadata;
}
exports.convertClientMetadataToStdMetadata = convertClientMetadataToStdMetadata;
// TODO: check if this results in sourcemap, if not, use new nodejs api if available
function convertErrorToMessage(error) {
    const stackAppendix = error.stack ? ` - ${error.stack}` : '';
    return `${error.message}${stackAppendix}`;
}
exports.convertErrorToMessage = convertErrorToMessage;
function formatMessage(message) {
    if (message instanceof Error) {
        return convertErrorToMessage(message);
    }
    return message;
}
exports.formatMessage = formatMessage;
function validateProjectId(projectId) {
    return constants_1.PATTERN_PROJECT_ID.test(projectId);
}
exports.validateProjectId = validateProjectId;
function createFullyQualifiedIdentifier(type, id, projectId) {
    if (typeof id !== 'string') {
        return undefined;
    }
    let fullyQualifiedIdentifier = id;
    const splits = fullyQualifiedIdentifier.split('/');
    if (splits.length === 4) {
        [, , , fullyQualifiedIdentifier] = splits;
    }
    if (type === 'traces') {
        if (!constants_1.PATTERN_TRACE_ID.test(fullyQualifiedIdentifier)) {
            // do nothing as it seems stackdriver accepts any value
            // throw new Error('Invalid traceId. Must be 32 hex chars.');
        }
    }
    // TODO: we could have a pattern for logNames
    return `projects/${projectId}/${type}/${fullyQualifiedIdentifier}`;
}
exports.createFullyQualifiedIdentifier = createFullyQualifiedIdentifier;
function formatHttpRequest(httpRequest) {
    var _a;
    if (typeof httpRequest === 'undefined') {
        return undefined;
    }
    let seconds = 0;
    let nanos = 0;
    const { latency } = httpRequest, rest = __rest(httpRequest, ["latency"]);
    if (typeof latency === 'undefined') {
        return rest;
    }
    if (typeof latency === 'number') {
        seconds = Math.floor(latency);
        nanos = Math.round((latency % 1) * Math.pow(10, 9));
    }
    else {
        latency.seconds = Math.floor((_a = latency.seconds) !== null && _a !== void 0 ? _a : 0);
        ({ seconds, nanos } = Object.assign({ seconds: 0, nanos: 0 }, latency));
    }
    return Object.assign(Object.assign({}, rest), { latency: {
            seconds,
            nanos,
        } });
}
exports.formatHttpRequest = formatHttpRequest;
function convertDurationToString(duration) {
    var _a, _b;
    let nanosString = '0';
    if (typeof duration === 'undefined') {
        return undefined;
    }
    if (typeof duration === 'number') {
        return `${duration}s`;
    }
    if (typeof duration.nanos === 'number' && duration.nanos > 0) {
        nanosString = `${(_a = String(mathjs_1.format(duration.nanos / Math.pow(10, 9), { notation: 'fixed' })).split('.')[1]) !== null && _a !== void 0 ? _a : 0}`;
    }
    const durationAsString = `${(_b = duration.seconds) !== null && _b !== void 0 ? _b : 0}.${nanosString}s`;
    return durationAsString;
}
exports.convertDurationToString = convertDurationToString;
function log(message, severity = 'log') {
    if (process.env.NODE_ENV !== 'production') {
        // eslint-disable-next-line no-console
        console[severity](message);
    }
}
exports.log = log;
function mergeLabels(defaultLabels = {}, entryLabels = {}) {
    const defaultKeys = Object.keys(defaultLabels);
    const entryKeys = Object.keys(entryLabels);
    if (defaultKeys.length === 0 || entryKeys.length === 0) {
        return Object.assign(Object.assign({}, defaultLabels), entryLabels);
    }
    const intersection = defaultKeys.filter((key) => entryKeys.includes(key));
    if (intersection.length > 0) {
        log(`Duplicate labels warning: Entry overwrites default metadata labels [${intersection.join(',')}]`, 'warn');
    }
    return Object.assign(Object.assign({}, defaultLabels), entryLabels);
}
exports.mergeLabels = mergeLabels;
//# sourceMappingURL=utils.js.map